if select(2, UnitClass("player")) ~= "SHAMAN" then
	return
end
-- do not load the addon if the current player is not a shaman

local SearingTotemTimer = LibStub("AceAddon-3.0"):NewAddon("SearingTotemTimer", "AceConsole-3.0")
local bars = DBT:New()

local defaults = {
	global = {
		barColor = RAID_CLASS_COLORS["SHAMAN"],
		textColor = {r=1, b=0, g=0}
	}
}

local sttEvent = "SPELL_CAST_SUCCESS"
local totemGUID = "DB00"
local summoned = 3599
local dismissed = 36936
local totemBar
local time = 60
local isSummoned

local timingLib = LibStub("SimpleTimingLib-1.0"):New()

local onSpellCast

local warningFrame = CreateFrame("Frame", nil, UIParent)
local warningFont = warningFrame:CreateFontString(nil, "OVERLAY", "ZoneTextFont")

do
	warningFrame:SetMovable(1)
	warningFrame:SetWidth(1)
	warningFrame:SetHeight(1)	
	warningFrame:SetFrameStrata("HIGH")
	warningFrame:SetClampedToScreen()
	warningFrame:SetPoint("TOP", 0, -300)
	warningFrame:Hide()
	warningFont:SetWidth(800)
	warningFont:SetHeight(0)
	warningFont:SetPoint("CENTER")
	warningFont:SetText("<< WARNING -- Searing Totem Expired -- WARNING >>")
	warningFont:SetTextHeight(27)
end

local function hideWrapper()
	warningFrame:Hide()
end

local function endWarning()
	warningFrame:Show()
	PlaySoundFile("Interface\\AddOns\\SearingTotemTimer\\Sounds\\AfflictionAlert.ogg")
	timingLib:Schedule(5, hideWrapper)
end

local function onEvent(self, event, ...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		--SearingTotemTimer:Print(...)
		local event = select(2, ...) -- get the subevent
		local sourceName, sourceFlags = select(5, ...) -- caster's name and flags
		local destFlag = select(9, ...)
		local spellId, spellName = select(10, ...) -- spell ID and name
		-- check whether the totem was summoned or dismissed
		if event == sttEvent then
			if spellId == summoned and bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) ~= 0 then
				onSpellCast()
			elseif spellId == dismissed and bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) ~= 0 and isSummoned then
					isSummoned = false
					stopCast()					
			end
		elseif event == "UNIT_DIED" and bit.band(destFlag, COMBATLOG_OBJECT_AFFILIATION_MINE) ~= 0 then
			stopCast()
		end
	end	
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
frame:SetScript("OnEvent", onEvent)

function onSpellCast()
	isSummoned = true
	local texture = select(3, GetSpellInfo(summoned))
	SearingTotemTimer.StartTimer(texture)
end

function SearingTotemTimer.StartTimer(texture)
	totemBar = bars:CreateBar(time, "SearingTotemBar", texture)
	totemBar:SetColor(SearingTotemTimer.db.global.barColor)
	timingLib:Schedule(time, endWarning)
end

function stopCast()
	bars:CancelBar("SearingTotemBar")
	timingLib:Unschedule(endWarning)
	isSummoned = false
end

function SearingTotemTimer.Unlock()
	bars:ShowMovableBar()
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:SetScript("OnEvent", function(self, event, addon)
	if addon == "SearingTotemTimer" then
		bars:LoadOptions("SearingTotemTimer")
	end
end)

local function createSetter(option)
	return function(info, value) bars:SetOption(option, value) end
end

local function createGetter(option)
	return function() return bars:GetOption(option) end
end

local options = {
	name = "SearingTotemTimer",
	type = "group",
	args = {
		options = {
			name = "Show the GUI",
			type = "execute",
			desc = "Loads the GUI for SearingTotemTimer options",
			func = function() -- show the GUI
				LibStub("AceConfigDialog-3.0"):Open("SearingTotemTimer")
			end,
			guiHidden = true -- don't show this in the GUI
		},
		unlock = {
			name = "Show movable bars",
			type = "execute",
			order = 0,
			func = SearingTotemTimer.Unlock
		},
		icon = {
			type = "select",
			name = "Icon position",
			order = 1,
			values = { -- lookup table to get a name (used for output) from the ID
				none = "Don't show icon",
				left = "Show icon on the left side",
				right = "Show icon on the right side",
				both = "Show icon on both sides"
			},
			get = function() -- translates from the two booleans into an ID
				if bars:GetOption("IconLeft") and bars:GetOption("IconRight") then
					return "both"
				elseif bars:GetOption("IconLeft") then
					return "left"
				elseif bars:GetOption("IconRight") then
					return "right"
				else
					return "none"
				end
			end,
			set = function(info, value) -- translates an ID into the two booleans
				if value == "none" then
					bars:SetOption("IconLeft", false)
					bars:SetOption("IconRight", false)
				elseif value == "left" then
					bars:SetOption("IconLeft", true)
					bars:SetOption("IconRight", false)
				elseif value == "right" then
					bars:SetOption("IconLeft", false)
					bars:SetOption("IconRight", true)
				elseif value == "both" then
					bars:SetOption("IconLeft", true)
					bars:SetOption("IconRight", true)
				end
			end
		},
		width = {
			name = "Bar width",
			type = "range",
			min = 100,
			max = 300,
			order = 2,
			get = createGetter("Width"),
			set = createSetter("Width")
		},
		scale = {
			name = "Bar scale",
			type = "range",
			min = 0.2,
			max = 2.0,
			order = 3,
			get = createGetter("Scale"),
			set = createSetter("Scale")
		},
		barColor = {
			name = "Bar Color",
			type = "color",
			order = 4,
			get = function()
				return SearingTotemTimer.db.global.barColor.r, SearingTotemTimer.db.global.barColor.g, SearingTotemTimer.db.global.barColor.b
			end,
			set = function(info, r, g, b)
				SearingTotemTimer.db.global.barColor = {r=tonumber(r), g=tonumber(g), b=tonumber(b)}
			end
		},
		textColor = {
			name = "Warning Text Color",
			type = "color",
			order = 5,
			get = function()
				return SearingTotemTimer.db.global.textColor.r, SearingTotemTimer.db.global.textColor.g, SearingTotemTimer.db.global.textColor.b
			end,
			set = function(info, r, g, b)
				textColor = {r=tonumber(r), g=tonumber(g), b=tonumber(b)}
				warningFont:SetTextColor(SearingTotemTimer.db.global.textColor.r, SearingTotemTimer.db.global.textColor.g, SearingTotemTimer.db.global.textColor.b)
			end
		}		
	}
}

LibStub("AceConfig-3.0"):RegisterOptionsTable("SearingTotemTimer", options, {"stt", "searingtotemtimer"})
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SearingTotemTimer")

function SearingTotemTimer:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SearingTotemTimerDB", defaults, true)
	warningFont:SetTextColor(SearingTotemTimer.db.global.textColor.r, SearingTotemTimer.db.global.textColor.g, SearingTotemTimer.db.global.textColor.b)
end