local MAJOR, MINOR = "SimpleTimingLib-1.0", 1
local SimpleTimingLib = LibStub:NewLibrary(MAJOR, MINOR)

if not SimpleTimingLib then
	return -- a greater or equal version is already loaded
end

SimpleTimingLib.tasks = SimpleTimingLib.tasks or {}
local tasks = SimpleTimingLib.tasks

local function schedule(time, func, obj, ...)
	local t = {...}
	t.func = func
	t.time = GetTime() + time
	t.obj = obj
	tasks[#tasks + 1] = t
end

local function unschedule(func, obj, ...)
	for i = #tasks, 1, -1 do
		local val = tasks[i]
		if val.obj == obj and (not func or val.func == func) then
			local matches = true
			for i = 1, select("#", ...) do
				if select(i, ...) ~= val[i] then
					matches = false
					break
				end
			end
			if matches then
				table.remove(tasks, i)
			end
		end
	end
end


local function onUpdate()
	for i = #tasks, 1, -1 do
		local val = tasks[i]
		if val and val.time <= GetTime() then
			table.remove(tasks, i)
			val.func(unpack(val))
		end
	end
end

SimpleTimingLib.frame = SimpleTimingLib.frame or CreateFrame("Frame")
local frame = SimpleTimingLib.frame
frame:SetScript("OnUpdate", onUpdate)

function SimpleTimingLib:Schedule(time, func, ...)
	return schedule(time, func, self, ...)
end

function SimpleTimingLib:Unschedule(func, ...)
	return unschedule(func, self, ...)
end

do
	local metatable = {__index = SimpleTimingLib}
	function SimpleTimingLib:New() -- constructor
		return setmetatable({}, metatable)
	end
end

function SimpleTimingLib_Schedule(time, func, ...)
	return schedule(time, func, nil, ...)
end

function SimpleTimingLib_Unschedule(func, ...)
	return unschedule(func, nil, ...)
end